module.exports = function (grunt) {
    'use strict';

    grunt.initConfig({
        pkg: grunt.file.readJSON('./package.json'),
        config: grunt.file.readJSON('./config.json'),

        copy: {
            single: {
                files: {
                    'dist/web/asset/js/packages/handlebars.min.js': ['./node_modules/handlebars/dist/handlebars.runtime.min.js'],
                    'dist/web/asset/js/packages/crossroads.min.js': ['./node_modules/crossroads/dist/crossroads.min.js'],
                    'dist/web/asset/js/packages/signals.min.js': ['./node_modules/crossroads/node_modules/signals/dist/signals.min.js'],
                    'dist/web/asset/js/packages/hasher.min.js': ['./bower_components/hasher/dist/js/hasher.min.js'],
                    'dist/web/asset/js/packages/spa.min.js': ['./bower_components/spa/dist/spa.min.js'],

                    'dist/web/asset/js/packages/jquery.min.js': ['./bower_components/jquery/dist/jquery.min.js'],
                    'dist/web/asset/js/packages/store.min.js': ['./bower_components/store-js/store.min.js'],


                    'dist/web/asset/js/packages/metro.min.js': ['./bower_components/metro/build/js/metro.min.js'],
                    'dist/web/asset/css/packages/metro.min.css': ['./bower_components/metro/build/css/metro.min.css'],
                    'dist/web/asset/css/packages/metro-icons.min.css': ['./bower_components/metro/build/css/metro-icons.min.css'],
                    'dist/web/asset/css/packages/metro-responsive.min.css': ['./bower_components/metro/build/css/metro-responsive.min.css'],
                    'dist/web/asset/css/packages/metro-schemes.min.css': ['./bower_components/metro/build/css/metro-schemes.min.css'],

                    'dist/web/emulator/asset/css/packages/pure.min.css': ['./bower_components/pure/pure-min.css'],
                    'dist/web/emulator/asset/js/packages/jquery.min.js': ['./bower_components/jquery/dist/jquery.min.js'],
                    'dist/web/emulator/asset/js/packages/crossroads.min.js': ['./node_modules/crossroads/dist/crossroads.min.js'],
                    'dist/web/emulator/asset/js/packages/signals.min.js': ['./node_modules/crossroads/node_modules/signals/dist/signals.min.js'],

                    'dist/web/emulator/asset/css/packages/animate.min.css': ['./bower_components/animate.css/animate.min.css'],
                }
            },
            metrofonts: {
                expand: true,
                cwd: './bower_components/metro/build/fonts/',
                src: ['**'],
                dest: 'dist/web/asset/css/fonts/'
            },
            ace: {
                expand: true,
                cwd: './bower_components/ace-builds/src-min-noconflict/',
                src: ['**'],
                dest: 'dist/web/asset/js/packages/ace/'
            },
            blockly: {
                expand: true,
                cwd: './bower_components/blockly/',
                src: ['blockly_compressed.js', 'blocks_compressed.js', 'javascript_compressed.js'],
                dest: 'dist/web/asset/js/packages/blockly/'
            },
            blocklyMedia: {
                expand: true,
                cwd: './bower_components/blockly/media/',
                src: ['**'],
                dest: 'dist/web/media/'
            },
            blocklyMsg: {
                expand: true,
                cwd: './bower_components/blockly/msg/',
                src: ['**'],
                dest: 'dist/web/asset/js/packages/blockly/msg/'
            },
            fontawesome: {
                expand: true,
                cwd: './bower_components/font-awesome/fonts/',
                src: ['**'],
                dest: 'dist/web/emulator/asset/fonts/'
            },
            dsl: {
                files: {
                    'build/dsl/tiledsl.js': ['src/tiledsl.js']
                }
            }
        },

        //minify all js files
        uglify: {
            options: {
                banner: '/*! <%= config.project.name %> v<%= config.project.version %> <%= grunt.template.today("yyyy-mm-dd") %> */\n',
                preserveComments: require('uglify-save-license')
            },
            dslcli: {
                files: [{
                    expand: true,
                    cwd: 'build/dsl',
                    src: '**/*.js',
                    dest: 'dist/dsl'
                }]
            },
            assets: {
                files: {
                    'dist/web/asset/js/lib.min.js': ['build/web/js/**/*.js', '!build/web/js/templates.js', '!src/web/blocks/**/*.js', '!build/web/js/data.js'],
                    'dist/web/asset/js/templates.min.js': ['build/web/js/templates.js'],
                    'dist/web/asset/js/customblocks.min.js': ['src/web/blocks/**/*.js'],
                    'dist/web/asset/js/main.min.js': ['build/web/js/data.js']
                }
            },
            emulator: {
                files: {
                    'dist/web/emulator/asset/js/packages/browserMqtt.min.js': ['node_modules/mqtt/browserMqtt.js'],
                    'dist/web/emulator/asset/js/tiles.min.js': ['build/emulator/js/**/*.js']
                }
            }
        },

        //compile .sass files to the css directory
        sass: {
            dist: {
                files: {
                    'build/web/css/theme/main.css': 'src/web/scss/main.scss',
                    'build/emulator/css/theme/main.css': 'src/emulator/style/main.scss',
                }
            }
        },

        //minify css files
        cssmin: {
            target: {
                files: [{//include all css files found in the css/ folder
                    expand: true,
                    cwd: 'build/web/css',
                    src: ['*.css', '!*.min.css'],
                    dest: 'dist/web/asset/css',
                    ext: '.min.css'
                }, {
                        'dist/web/asset/css/theme/main.min.css': ['build/web/css/theme/main.css']
                    }]
            },
            emulator: {
                files: [{
                    expand: true,
                    cwd: 'build/emulator/css',
                    src: ['*.css', '!*.min.css'],
                    dest: 'dist/web/emulator/asset/css',
                    ext: '.min.css'
                }, {
                        'dist/web/emulator/asset/css/theme/main.min.css': ['build/emulator/css/theme/main.css'],
                        'dist/web/emulator/asset/css/main-grid.min.css': ['build/web/css/pure-grid.css']
                    }]
            }
        },

        pure_grids: {
            responsive: {
                dest: './build/web/css/pure-grid.css',

                options: {
                    units: 12, // 12-column grid

                    mediaQueries: {
                        sm: 'screen and (min-width: 35.5em)', // 568px
                        md: 'screen and (min-width: 48em)',   // 768px
                        lg: 'screen and (min-width: 64em)',   // 1024px
                        xl: 'screen and (min-width: 80em)'    // 1280px
                    },
                    selectorPrefix: '.col-'
                }
            }
        },

        //assemble all .hbs files in views/assemble
        assemble: {
            options: {
                data: ['config.json'],
                layout: 'simple.hbs',
                layoutdir: './src/web/views/assemble/layouts',
                partials: './src/web/views/assemble/partials/**/*.hbs'
            },

            //sidebar page
            sidebar: {
                options: {
                    layout: 'base.hbs'
                },
                files: [{
                    cwd: './src/web/views/assemble/pages/',
                    dest: 'build/web/',
                    expand: true,
                    src: '**/*.hbs'
                }]
            },

            emulator: {
                options: {
                    layout: 'base.hbs',
                    layoutdir: './src/emulator/view/layouts',
                    partials: './src/emulator/view/partials/**/*.hbs'
                },
                files: [{
                    cwd: './src/emulator/view/pages/',
                    dest: 'build/emulator/',
                    expand: true,
                    src: '**/*.hbs'
                }]
            }
        },

        //compile handlebars templates to js
        handlebars: {
            compile: {
                options: {
                    namespace: "<%= config.namespace %>",
                    partialsUseNamespace: true,
                    partialsPathRegex: /\/partials\//,
                    partialRegex: /.*\.hbs/,

                    processName: function (fileName) {
                        return fileName.replace("src/web/views/templates/", '').replace(".hbs", '');
                    },

                    processPartialName: function (fileName) {
                        return fileName.replace("src/web/views/templates/partials/", '').replace(".hbs", '');
                    }
                },
                files: {
                    "build/web/js/templates.js": "src/web/views/templates/**/*.hbs"
                }
            }
        },

        htmlmin: {
            dist: {
                options: {
                    removeComments: true,
                    collapseWhitespace: true
                },
                files: [{
                    cwd: './build/web/',
                    dest: 'dist/web/',
                    expand: true,
                    src: '**/*.html'
                }]
            },
            emulator: {
                options: {
                    removeComments: true,
                    collapseWhitespace: true
                },
                files: [{
                    cwd: './build/emulator/',
                    dest: 'dist/web/emulator/',
                    expand: true,
                    src: '**/*.html'
                }]
            }
        },

        ts: {
            cli: {
                src: ['src/dsl/**/*.ts'],
                outDir: 'build/dsl/',
                tsconfig: 'src/dsl/tsconfig.json',
                options: {
                    sourceMap: true,
                    declaration: true
                }
            }
        },

        webpack: {
            emulator: {
                entry: './src/emulator/tiles/main.ts',
                output: {
                    filename: './build/emulator/js/bundle.js'
                },
                resolve: {
                    // Add `.ts` and `.tsx` as a resolvable extension.
                    extensions: ['', '.webpack.js', '.web.js', '.ts', '.tsx', '.js']
                },
                module: {
                    loaders: [
                        // all files with a `.ts` or `.tsx` extension will be handled by `ts-loader`
                        { test: /\.tsx?$/, loader: 'ts-loader' }
                    ]
                }
            },
            web: {
                entry: './src/web/main.ts',
                output: {
                    filename: './build/web/js/bundle.js'
                },
                resolve: {
                    // Add `.ts` and `.tsx` as a resolvable extension.
                    extensions: ['', '.webpack.js', '.web.js', '.ts', '.tsx', '.js']
                },
                module: {
                    loaders: [
                        // all files with a `.ts` or `.tsx` extension will be handled by `ts-loader`
                        { test: /\.tsx?$/, loader: 'ts-loader' }
                    ]
                },
                node: {
                    fs: "empty"
                }
            }
        },


        json: {
            main: {
                options: {
                    namespace: 'Data',
                    includePath: true,
                    processName: function (filename) {
                        return filename.toLowerCase().replace(".json", "");
                    }
                },
                src: ['config.json'],
                dest: 'build/web/js/data.js'
            }
        },

        connect: {
            server: {
                options: {
                    port: 8081,
                    base: './dist/web/',
                    keepalive: true
                }
            }
        }
    });

    /* DSL CLI*/
    grunt.loadNpmTasks('grunt-contrib-uglify');
    grunt.loadNpmTasks("grunt-ts");
    /* TDE */
    grunt.loadNpmTasks('grunt-assemble');
    grunt.loadNpmTasks('grunt-contrib-sass');
    grunt.loadNpmTasks('grunt-contrib-cssmin');
    grunt.loadNpmTasks('grunt-contrib-connect');
    grunt.loadNpmTasks('grunt-contrib-handlebars');
    grunt.loadNpmTasks('grunt-contrib-copy');
    grunt.loadNpmTasks('grunt-contrib-htmlmin');
    grunt.loadNpmTasks('grunt-pure-grids');
    grunt.loadNpmTasks('grunt-json');

    grunt.loadNpmTasks("grunt-webpack");


    /* grunt tasks */
    grunt.registerTask('compile', ['assemble', 'pure_grids', 'sass', 'cssmin', 'handlebars', 'ts', 'webpack', 'json', 'copy', 'uglify', 'htmlmin']);
    grunt.registerTask('runWeb', ['connect']);
    grunt.registerTask('compileWeb', ['assemble:sidebar', 'sass', 'cssmin', 'handlebars', 'webpack:web', 'json', 'uglify:assets', 'copy', 'htmlmin:dist']);
    grunt.registerTask('compileDSL', ['ts:cli', 'copy:dsl', 'uglify:dslcli']);
    grunt.registerTask('compileEmulator', ['assemble:emulator', 'webpack:emulator', 'uglify:emulator', 'sass', 'cssmin', 'htmlmin:emulator']);
};