# TILE Development Toolkit
This project consist of three parts: a Domain-Specific-Language for TILES, a web-based development environment for creating applications for TILES, and an emulator for emulationg TILE devices.

The project is part of the TILES project at the [Norwegian University of Science and Technology](https://www.ntnu.no/).


## Tiles Web-Client

Tiles Web-Client is web-envioronment for configuring your `TILES`. Create application for interacting with your `TILE` devices, visually and textually.

## Tile DSL

This project implements a DSL for creating TILE applications. It consist of `modules`, made available in a TILE DSL interface, which is callable by the DSL language. The project is built using `TypeScript`.

## Tile Emulator

Tile emulator which can be run in a web-browser

## Setup

In order to build Tiles Development Toolkit, ensure that you have [Node.js](https://nodejs.org/), [Bower](http://bower.io/), [Typings](https://github.com/typings/typings), [TypeScript](http://www.typescriptlang.org/), [Grunt](http://gruntjs.com/), [PEG.js](https://www.npmjs.com/package/pegjs), and [WebPack](https://www.npmjs.com/package/webpack) globally installed.
    
    npm install -g typings
    npm install -g bower
    npm install -g grunt-cli
    npm install -g typescript
    npm install -g pegjs
    npm install  -g webpack 


* Install dependencies
```sh
npm install
bower install
typings install
```


## Building

The build tool used is Grunt, a dependency that already should be installed locally on your system.


All available tasks are defined in `Gruntfile.js`. To run the tasks, simply use
    grunt <buildname>

There are three different tasks registered with grunt:
* `compile` - compiles all three project `dsl`, `web-env` and `emulator`
* `runWeb` - run web environment
* `compileWeb` - Compiles web-environment
* `compileDSL` - Compiles DSL 
* `compileEmulator` - Compiles emulator

The npm command can also be used to start the build process. This will delete the `build/` and `dist/` directory, and run the grunt `compile` task.


A successful build will create two folder: ´build/´ and ´dist/´, as described in [the various directories](#directories)


### Visual Studio Code

The [Visual Studio Code](https://code.visualstudio.com/) editor can be configured to run a Grunt task on a given keyboard shurtcut. Standard shortcut is `ctrl`+`shift`+`b`. The tasks.json configures the building shortcut, and runs the ´grunt compile´ task.

here the builds tasks is defined as `compile`, defined in `Gruntfile.js`. 

## Running
Running the `grunt runWeb` command in the terminal, will start a server, making the files within the `dist/web` folder accessible from a web-browser. Navigate your web-browser to [localhost:8081](http://localhost:8081/).

When packaged, the web-application consist of a single `.html` page, used as a Single-Page-Application layout, which loads pages dynamically. All dependencies are located in `dist/asset/`, where both `.css` and `.js` files are located. Packages from `bower` and `npm` are located under `packages/` folders. 



To run the dsl package

```ssh
npm run start
```

Running the unminified version of the dsl package

```ssh
npm run debug
```


Creating the pegjs file (requires `pegjs` to be installed globally. See the `Setup` section for detail)

```ssh
npm run peg
```


To rebuilt the project, which will delete both the `build/` and `dist/` folder, and compile the source code

```ssh
npm run clean
```


## <a name="directories"></a>The various directories

* `src`: contains the source code of the project
    * `modules`: .ts code used for each module defined in the project. Contains the main file, that loads the project
    * `scss`: sass style defined for the project
    * `views`: handlebars layouts defined. Contains both `assemble.io` files, and handlebars files that are compiled to a .js file 
* `typings`: holds the typings files `*.d.ts` defined in `typings.json`
* `build`: files that are compiled from the `src` folder
* `dist`: production ready code, created from the dist folder





## DSL language

The DSL language is defined using [pegjs](http://pegjs.org/), and located in `src/tiledsl.pegjs`. This definition is compiled to a JavaScript object, using the `pegjs` node package.


Compilation of the DSL is done using the pegjs object, where `dslCode` is a variable containing a script in the DSL language.

```js
syntax=require('./tiledsl');
var jsCode=syntax.parse(dslCode);
```

The variable `jsCode` will now contain the JavaScript script representation of the DSL.



## DSL Functionalities

The DSL provides multiple functions that are available to its users. Creating an applications for TILES can be done by plain `JavaScript` code, or the described functions below.

Before the application is ready and can be run, the user needs to be fetched from the database. By supplying a username, this is done automaticly by the DSL.


### MQTT

The DSL provides `MQTT` capabilities for communicating with the TILEs registered to the current user. The application are able to listen for events triggered on a given TILE, or send commands to a specific TILE. 

All TILES are available by using the built-in function

```js
var myTiles = this.tiles;
```

To retrieve a specific TILE, its id is required

```js
var alphaTile = this.tiles["alpha"];
```

#### Send Commands

Sending a command to a tile requires the user to specify the command which should be sent. This need to conform with the command interface defined in `src/model/interface/iCommandts`.

```js
var command=new Command({type:"led",activation:"on",duration:1000,pattern:"normal"});
this.mqtt.sendCommand(alphaTile,command);
```

#### Receive Events

Received events on a user registered TILE will trigger an event on the Tile object. To setup a listener for a tile and a given event, where `data` is the event details (see `iEvent` in `src/model/interface/iEvent.ts`)

```js
alphaTile.on('tap',function(data){
    //TODO: do some action here
});
```




### HTTP

Fetching or sending data to a third-party service requires the use of the HTTP function. Both what data to send, and headers required by the third-party service can be defined by the http object. To start a new http connection, the desired method (`get`, `post`, `put`, `delete`) needs to be specified, along with the `url`. To pass data to the http request, the function `formData()` should be used, which take a input param of an object. The same case is for setting header data, using `headers()`. Either function can be left out, if these should be left empty.

```js
this.http(url,method).formData({}).headers({}).exec(function(data){
});
```


### User



### EventEmitter



### TILE



## Dependencies

Tiles Development Toolkit is built with multiple libraries.

* [Metroui](http://metroui.org.ua/) - ui framework built with css and js
* [Spa](https://github.com/jonaskirkemyr/spa) - Single-Page-Application Typescript Library
    * [Crossroads](https://millermedeiros.github.io/crossroads.js/) - Javascript routing library
    * [Hasher](https://github.com/millermedeiros/Hasher) - Browser history manager
* [Assemble](http://assemble.io/) - Static compilation of Handlebars template 
* [Handlebars](http://handlebarsjs.com/) - extension of the Mustache templating language, a logicless templating language
* `Pure`
* `Font-awesome`
* `JQuery`

## License

Tile Development Toolkit is licensed under `Apache-2.0`. See [LICENSE](LICENSE) for more information. 