import tile = require('tile');


//COMMANDS
const COMMANDS = {
    SPEAKER: {
        PROP1: ["start", "stop", "pause"],
        /*ACTIVATION: ["on", "off"],
        PATTERN: ["normal"]*/
    },
    LED: {
        PROP1: ["on", "off", "blink", "fade"],
        PROP2: ["red", "green", "blue", "white"]
        /*ACTIVATION: ["on", "off"],
        PATTERN: ["normal", "blink", "fade", "flash"]*/
    },
    VIBRATE: {
        PROP1: ["long", "burst"],
    }
};


class Command {
    private cmnd: tile.iCommand;

    /**
    * Validates a command
    * 
    * @static
    * @param {Command} cmnd the command to validate
    * @returns {boolean} whether input command is valid or not
    */
    private static validate(cmnd: tile.Message): boolean {
        if (cmnd.name == undefined || cmnd.properties == undefined)
            return false;

        var name = cmnd.name.toUpperCase();
        var command = COMMANDS[name];

        return (
            (name in COMMANDS) &&
            command.PROP1.indexOf(cmnd.properties[0]) >= 0 &&
            (command.PROP2 != undefined && (command.PROP2.indexOf(cmnd.properties[1]) >= 0))
        );
    }

    /**
     * Parses an object to check if its a
     * valid iCommand
     *
     * @static
     * @param {Object} cmnd object to validate and parse
     * @returns {Command} validated Command
     */
    public static parse(cmnd: tile.Message): Command {
        var temp: Command = null;
        var test = cmnd.properties[0];
        if (this.validate(cmnd))
            temp = new Command({ name: <tile.outputInterfaces>cmnd.name, event: <tile.commandEvents>cmnd.properties[0], param: cmnd.properties[1] })
        return temp;
    }


    constructor(cmnd: tile.iCommand) {
        this.cmnd = cmnd;
    }

    get name() {
        return this.cmnd.name;
    }

    get event() {
        return this.cmnd.event;
    }

    get param() {
        return this.cmnd.param;
    }

    set param(str: string) {
        this.cmnd.param = str;
    }

    /**
     * String representation of iCommand
     *
     * @returns {string} stringified object
     */
    public toString(): string {

        var prop: Array<string> = [];
        if (this.cmnd.event != undefined)
            prop.push(this.cmnd.event);
        if (this.cmnd.param != undefined)
            prop.push(this.cmnd.param);
        var temp = { name: this.cmnd.name, properties: prop };
        return JSON.stringify(temp);
    }

}

export = Command;