import astring = require('astring');
import TwitterOpts = require('./TwitterOpts');

import dsl = require('./iTileDsl');
import fs = require('fs');

const PIPENAME = "pipe";


/**
 * Class for parsing a AST to JavaScript code
 * 
 * @class Parser
 */
class Parser {
    private _ast: ESTree.Node | ESTree.Node[] = null;//holds the AST 
    private _generator: dsl.TileDsl = <dsl.TileDsl>astring.defaultGenerator;

    private astringOptions: astring.State;

    public static twitterOpts = new TwitterOpts();

    public static pipeNumber = 0;

    public static self = "dsl";

    public static lastBlockPosition = null;//at which position to automatically place a reset token

    private compiledCode: string = null;

    public constructor() {
        // add generator methods for recognizing DSL functionalities
        this.initGenerator();

        this.astringOptions = {
            indent: '    ',
            lineEnd: '\n',
            generator: this._generator
        }
    }

    get ast(): ESTree.Node | ESTree.Node[] {
        return this._ast;
    }

    set ast(ast: ESTree.Node | ESTree.Node[]) {
        this._ast = ast;
    }

    private initGenerator() {
        this._generator.MainStatement = function (node, state) {
            state.output.write("main = function(" + Parser.self + ")");
            this[node.consequent.type](node.consequent, state);
            state.output.write(";" + state.lineEnd);
            state.output.write("return output(options,main)");//return main and options function
        }

        this._generator.OptionStatement = function (node, state) {
            state.output.write("options=function(" + Parser.self + ")");
            this["BlockStatement"](node, state);
            state.output.write(";");
            state.output.write(state.lineEnd);
        }

        /** OPTIONS */
        this._generator.ConsumerSecret = function (node, state) {
            if (node.init != null) {
                Parser.twitterOpts.consumerSecretKey = <string>node.init.value;

                if (Parser.twitterOpts.isReady)
                    state.output.write(Parser.twitterOpts.write);
            }
        }
        this._generator.ConsumerKey = function (node, state) {
            if (node.init != null) {
                Parser.twitterOpts.consumerKey = <string>node.init.value;

                if (Parser.twitterOpts.isReady)
                    state.output.write(Parser.twitterOpts.write);
            }
        }
        this._generator.Token = function (node, state) {
            if (node.init != null) {
                Parser.twitterOpts.token = <string>node.init.value;

                if (Parser.twitterOpts.isReady)
                    state.output.write(Parser.twitterOpts.write);
            }
        }
        this._generator.TokenSecret = function (node, state) {
            if (node.init != null) {
                Parser.twitterOpts.tokenSecret = <string>node.init.value;

                if (Parser.twitterOpts.isReady)
                    state.output.write(Parser.twitterOpts.write);
            }
        }

        this._generator.FacebookAPI = function (node, state) {
            if (node.init != null)
                state.output.write(Parser.self + ".initFacebook(\'" + node.init.value + "\');");
        }
        this._generator.CustomHTTP = function (node, state) {
            if (node.init != null) {
                state.output.write("customhttp.options.");
                this[node.init.type](node.init, state);
            }

        }
        this._generator.WeatherAPI = function (node, state) {
            if (node.init != null)
                state.output.write(Parser.self + ".initWeather(\'" + node.init.value + "\');");
        }
        /** END OPTIONS */


        /** VARIABLES */
        this._generator.facebookvar = function (node, state) {
            if (node.option != null)
                state.output.write(Parser.self + ".facebook." + node.option + "()");
        }
        this._generator.twittervar = function (node, state) {
            if (node.option != null)
                state.output.write(Parser.self + ".twitter." + node.option + "()");
        }
        this._generator.weathervar = function (node, state) {
            if (node.option != null && node.body != null) {

                state.output.write(Parser.self + ".weather." + node.option + "(function(weather)");
                this[node.body.type](node.body, state);

                state.output.write(");");
            }
        }
        this._generator.mevar = function (node, state) {
            state.output.write(Parser.self + ".user" + (node.option != null) ? "." + node.option : null);
        }
        this._generator.tile = function (node, state) {
            if (node.name != null)
                state.output.write(Parser.self + ".tiles." + node.name);
        }
        this._generator.tiles = function (node, state) {
            state.output.write(Parser.self + ".tiles");
        }
        /** END VARIABLES */

        /** STATEMENTS */
        this._generator.DslIf = function (node, state) {
            if (node.event != null) {
                if (node.event.tile != null) {
                    this[node.event.tile.type](node.event.tile, state);

                    state.output.write(".once(\"" + node.event.event + "\", function(event){");
                    if (node.event.param != null)
                        state.output.write("if(event==\"" + node.event.param + "\")")
                }

                if (node.body != null) {
                    this["BlockStatement"](node, state);
                    Parser.computeLastBlockPosition(state);
                }

                state.output.write("});");
            }
        }
        this._generator.DslIfSimple = function (node, state) {
            //assign the command to the body..
            node["body"] = [node.command];
            //to use it with existing if statement
            this["DslIf"](node, state);
        }
        this._generator.Repeat = function (node, state) {
            if (node.number != null) {
                state.output.write("for(var i=0;i<");
                this[node.number.type](node.number, state)
                state.output.write(";++i)");
                if (node.body != null) {
                    this["BlockStatement"](node, state);
                    Parser.computeLastBlockPosition(state);
                }
            }
        }
        this._generator.Sync = function (node, state) {
            if (node.events != null) {
                var pipeVar = "pipe" + Parser.pipeNumber;// pipe variable and event listener name

                state.output.write(Parser.self + ".initPipe(\"" + pipeVar + "\"");

                //  state.output.write("var " + pipeVar + "=new Pipe(\'" + pipeVar + "\');");//create variable

                //add eventfilters for each event and tile given
                node.events.forEach(element => {
                    state.output.write(state.lineEnd + state.indent + state.indent);
                    state.output.write(",{");

                    state.output.write("tile:");
                    this[element.tile.type](element.tile, state);//output tile
                    state.output.write(",event:{");
                    state.output.write("name:\"" + element.event + "\"");
                    if (element.param != null)
                        state.output.write(",event:\"" + element.param + "\"");
                    state.output.write("}}");


                });
                state.output.write(");");

                //add code to run when pipe is triggered 
                if (node.body != null) {
                    state.output.write(state.lineEnd + state.indent);
                    state.output.write(Parser.self + ".once(\"" + pipeVar + "\",function()");
                    this["BlockStatement"](node, state);
                    Parser.computeLastBlockPosition(state);
                    state.output.write(");");
                }
                ++Parser.pipeNumber;//increase the pipenumber, to prevent collision of pipe listeners
            }
        }
        this._generator.Random = function (node, state) {
            if (node.random != null) {

                state.output.write(Parser.self + ".random(");
                this[node.random.type](node.random, state);
                state.output.write(")");
            }
        }
        this._generator.Reset = function (node, state) {
            state.output.write("return main(dsl);");
            state.output.write(state.lineEnd + state.indent);

            Parser.lastBlockPosition = undefined;//set last block as undefined, making sure an automatic insert on lastblockposition won't occur
        }
        this._generator.TileEvent = function (node, state) {
            if (node.event != null) {//check that an event is given 
                var param = (node.param != null) ? "." + node.param : "";//whether an extra param is given for event

                if (node.tile != null) {//check that a tile is given
                    this[node.tile.type](node.tile, state);
                    state.output.write("." + node.event + param);
                }
            }
        }
        this._generator.TileColor = function (node, state) {
            if (node.color != null)//check whether color is set
                state.output.write(node.color);
        }
        this._generator.TileCommand = function (node, state) {
            if (node.command != null) {
                state.output.write(Parser.self + "." + node.command.type + ".");// + node.command.command + "(");

                var func = null;

                if (node.command.command == "led") {
                    if (node.param != null)
                        func = (node.param == "on") ? "turnOn" : "turnOff";
                    else
                        func = "turnOn";
                }
                else if (node.command.command == "vibrate") {
                    if (node.param != null)
                        func = (node.param == "on") ? "start" : "stop";
                    else
                        func = "start";
                }
                else
                    func = node.command.command;

                state.output.write(func + "(");

                if (node.tile != null)
                    this[node.tile.type](node.tile, state);
                /*if (node.color != null) {
                    state.output.write(",\"");
                    this[node.color.type](node.color, state);
                    state.output.write("\"");
                }*/
                state.output.write(");");
            }
        }

        this._generator.DatasourceCommand = function (node, state) {
            //check for valid datasource command

            var ds = node.ds.toLowerCase();
            var method = node.method;

            state.output.write(Parser.self+".");
            if (ds == "twitter") {
                state.output.write("twitter");
                if (method == "tweet") {
                    var param = "status:";
                    state.output.write(".tweet({");
                    state.output.write("status:");
                    this[node.value.type](node.value, state);
                    state.output.write("},function(data){});");
                }
            }
            else {
                state.output.write(ds + "." + node.method + "(");
                this[node.value.type](node.value, state);
                state.output.write(");");

            }
            /* switch (node.ds.toLowerCase()) {
                 case 'facebook':
                 case 'twitter':
                 case 'customhttp':
                     break;
                 default:
                     throw new Error("DS not recognized: " + node.ds);
             }
 
             output += node.ds.toLowerCase();
 
             if (node.method !== null)
                 output += "." + node.method + "(";
 
             state.output.write(output);
             this[node.value.type](node.value, state);*/
            //state.output.write(");");

        }
        /** END STATEMENTS */
    }



    /**
     * Generate JavaScript code from an AST
     * 
     * @type {string}
     */
    public compile() {

        if (this._ast == null)
            return "Error: No AST provided";
        var code = "var main=undefined;\nvar options=undefined;\n";

        if (this._ast instanceof Array)
            (<ESTree.Node[]>this._ast).forEach(element => {
                code += this.insertLastBlock(astring(element, this.astringOptions));
            });
        else {
            code += this.insertLastBlock(astring(<ESTree.Node>this._ast, this.astringOptions));
        }

        this.compiledCode = code;
    }

    public get code(): string {
        return this.compiledCode;
    }

    public toFile(path: string) {
        fs.writeFileSync(path, this.code);
    }

    public static computeLastBlockPosition(state: astring.State) {
        if (this.lastBlockPosition !== undefined)//will only be undefined if a Reset token exists (wouldn't want to include Reset if already present)
            this.lastBlockPosition = state.output.toString().length - 2;
    }

    private insertLastBlock(code) {
        var pos = Parser.lastBlockPosition;
        if (pos != null || pos != undefined) {
            var insert = astring({ "type": "Reset" }, this.astringOptions);//code to insert at last block, to make the application repeat itself
            var temp = [code.slice(0, pos + 1), insert, code.slice(pos)].join("");
            code = temp;
        }
        return code;
    }
}

export = Parser;