import tile = require('tile');
//EVENTS
const EVENTS = {
    TAP: {
        SINGLE: "single",
        DOUBLE: "double",
        TOUCH_HOLD: "touch_hold"
    },

    TILT: {
        LEFT: "left",
        RIGHT: "right",
        UPDOWN: "updown"
    },

    ROTATE: {
        CLOCKWISE: "clockwise",
        ANTICLOCKWISE: "anticlockwise"
    },

    SHAKE: {
        HORIZONTALLY: "horizontally",
        VERTICALLY: "vertically"
    },

    VERTICAL_SHIFT: {
        LIFT: "lift",
        FREEFALL: "freefall"
    },

    PROXIMITY: {
        LEFT: "left",
        RIGHT: "right",
        TOP: "top",
        BOTTOM: "bottom"
    }
};

class Event {
    private evt: tile.iEvent;

    /**
     * Validates an event
     * 
     * @static
     * @param {Event} evt the event to validate
     * @returns {boolean} whether input event is valid or not
     */
    private static validate(evt: tile.Message): boolean {
        if (evt.name == undefined || evt.properties == undefined || evt.properties.length == 0)
            return false;

        var event = evt.properties[0].toUpperCase();
        var name = evt.name.toLowerCase();
        switch (name) {
            case "tap":
                return (event in EVENTS.TAP);
            case "tilt":
                return (event in EVENTS.TILT);
            case "rotate":
                return (event in EVENTS.ROTATE);
            case "shake":
                return (event in EVENTS.SHAKE);
            case "vertical_shift":
                return (event in EVENTS.VERTICAL_SHIFT);
            case "proximity":
                return (event in EVENTS.PROXIMITY);
        }

        return false;
    }


    /**
     * Parses an object to check if its a
     * valid iEvent
     * 
     * @static
     * @param {JSON} evt object to validate and parse
     * @returns {Event} validated Event
     */
    static parse(evt: tile.Message): Event {
        var event: Event = null;
        if (this.validate(evt))
            event = new Event({ name: <tile.eventNames>evt.name, event: <tile.event>evt.properties[0], param: evt.properties[1] });
        return event;
    }

    public constructor(evt: tile.iEvent) {
        this.evt = evt;
    }

    get name() {
        return this.evt.name;
    }

    get event() {
        return this.evt.event;
    }

    get param() {
        return this.evt.param;
    }

    /**
     * String representation of iEvent
    * 
    * @returns {string} stringified object
    */
    public toString(): string {
        var prop:Array<string>=[];
        if(this.evt.event!=undefined)
            prop.push(this.evt.event);
        if(this.evt.param!=undefined)
            prop.push(this.evt.param);
        var temp = { name: this.evt.name, properties: prop };
        return JSON.stringify(temp);

    }
}

export = Event;