import minimist = require('minimist');
import chalk = require('chalk');
import fs = require('fs');


const args = minimist(process.argv.slice(2));//retrieve arguments 
//retrieve arguments
var username = args["u"] || args["username"]; //-u <username> | --user=<username>           username to run application for
var run: boolean = args["r"] || args["run"]; //-r | --run=<bool>                            Whether code should be run immediately after compilation
var code: string = args["c"] || args["code"]; //-c | --code "this is my long code:)"        Code to parse
var debug: boolean = args["d"] || args["debug"]; //-d | --debug                              Will print status to console
var files: Array<string> = args["_"];

var input:string=null;
var output:string=null;

if (code == undefined) {
    if (files.length == 0) {
        console.log("ERROR: No input files given");
        process.exit(0);
    }

    input = files[0];//input .dsl file
    output = (files.length > 1) ? files[1] : "output.js";//output .js file

    //read given input file

    try {
        code = fs.readFileSync(input, { encoding: "utf8" });
    }
    catch (e) {
        if (debug) {
            console.log(chalk.red("ERROR:"));
            console.log(chalk.red("Couldn't open file ") + chalk.magenta(input));
            console.log(chalk.red("Does the file exist?"));
        }
        process.exit(0);
    }
}

//Parsing dsl code
import Parser = require('../DslParser/Parser');
var parse = new Parser();
var syntax = require('../tiledsl');

try {
    parse.ast = syntax.parse(code);//parse code, create ast
    parse.compile();
    if(output!=null)
        parse.toFile(output);//write compiled code to file
    if (debug && input!=null && output != null) {
        console.log(chalk.green("Success: dsl code compiled successfully"));
        console.log("\t" + chalk.cyan(input) + " -> " + chalk.cyan(output));
    }
}
catch (e) {
    if (debug) {
        console.log(chalk.red("Compile Error:"));
        console.log(chalk.red(e));
    }
    process.exit(0);
}

// If the code should not be run..
if (!run)
    process.exit(0);//just exit 


import mongoose = require('mongoose');
//load models
var Tile = require("./mongoose/Tile");
var User = require("./mongoose/User");
import Config = require('./Config');
import UserDSL = require('./UserDSL');

var Spinner = require('cli-spinner').Spinner;

Config.username = username;


//connect to db
mongoose.connect(Config.db);

var dsl = new UserDSL();
if (debug)
    dsl.debug = console.log;
try {
    dsl.userFunction = new Function('output', parse.code);
    dsl.on("ready", function () {
        if (debug) {
            console.log(chalk.cyan("Start running DSL code"));
            var spinner = new Spinner(chalk.cyan("%s"));
            spinner.setSpinnerString('|/-\\');
            spinner.start();
        }
        dsl.run();
    });
}
catch (e) {
    if (debug) {
        console.log(chalk.red("Run time Error:"));
        console.log(chalk.red(e));
    }
    process.exit(0);
}


