import Events = require('events');

/**
 * EventEmitter class that should be extended
 * by classes which requires an EventEmitter
 * 
 * This class only define functions that should
 * be made available public
 * 
 * @abstract
 * @class EventEmitter
 */
abstract class EventEmitter {
    /**
     * The eventemitter instance
     * 
     * @protected
     * @type {Events.EventEmitter}
     */
    private event: Events.EventEmitter = null;

    /**
     * Creates an instance of EventEmitter.
     */
    public constructor() {
        this.event = new Events.EventEmitter();
    }
    /**
     * Emits new event on object
     * 
     * @see Events.EventEmitter.emit 
     * 
     * @param {string} event to trigger on object
     * @param {...any[]} args arguments which should be passed to function
     * @returns {boolean} whether emit was successful or not
     */
    public emit(event: string, ...args: any[]): boolean {
        return this.event.emit(event, args);
    }
    /**
     * Listen for event triggered on object
     * 
     * @see Events.EventEmitter.on
     * 
     * @param {string} event to listen for
     * @param {Function} listener function to run when called
     * @returns {Events.EventEmitter} 
     */
    public on(event: string, listener: Function): Events.EventEmitter {
        return this.event.on(event, listener);
    }
    /**
     * Listen for event and trigger function
     * only once
     * 
     * @param {string} event to listen for
     * @param {Function} listener function to run when called
     * @returns {Events.EventEmitter}
     */
    public once(event: string, listener: Function): Events.EventEmitter {
        return this.event.once(event, listener);
    }
}
export = EventEmitter;