import iFilter = require('./iFilter');

class Pipe {
    private _name: string;

    private filters: Array<iFilter>;

    private pos: number;//the position of the current filter

    public constructor(name: string) {
        this._name = name;
        this.pos = 0;
        this.filters = [];
    }

    public add(filter: iFilter): Pipe {
        this.filters.push(filter);
        return this;
    }

    public execute(data: any): boolean {
        if (this.pos < 0 || this.pos >= this.filters.length)
            return false;
        if (!this.filters[this.pos].run(data)) {//if filter didn't pass execution data
            this.resetFilters();
            return false;
        }

        ++this.pos;//increase, go to the next filter

        if (this.pos == this.filters.length) {
            this.resetFilters();
            return true;
        }


        return false;
    }

    private resetFilters() {
        for (var i = this.pos - 1; i >= 0; --i)//reset all previous filters
            this.filters[i].status = false;
        this.pos = 0;
    }

    get name(): string {
        return this._name;
    }

}

export = Pipe;