import HTTPClient = require('../HTTPClient');
import ErrorHandler = require('../../../ErrorHandler');

import iFacebookInterfaces = require('../Facebook/iFacebookInterfaces');

class Facebook {
    private http: HTTPClient = null;

    private baseurl: string = "https://graph.facebook.com/v2.6";
    private apiKey: string = null;
    private userId: string = null;

    public constructor(apiKey: string) {
        this.apiKey = apiKey;
    }

    private initHttp(url: string, type: "get" | "post" | "put" | "delete"): HTTPClient {
        var http = new HTTPClient(this.baseurl + url, type);

        http.queries = {
            "access_token": this.apiKey
        }

        return http;
    }

    /**
     * Retrieves posts posted by current user
     * 
     * @param {(data: iFacebookInterfaces.iResult<iFacebookInterfaces.iPost>) => void} func the function which should retrieve the returned data
     */
    public posts(func: (data: iFacebookInterfaces.iResult<iFacebookInterfaces.iPost>) => void) {
        this.initHttp("/me/posts", "get").exec(function (data: iFacebookInterfaces.iResult<iFacebookInterfaces.iPost>) {
            func(data);
        });
    }

    /**
     * Retrievse the wall/feed for current user
     * 
     * @param {(data: iFacebookInterfaces.iResult<iFacebookInterfaces.iFeed>) => void} func the function which should retrieve the returned data
     */
    public feed(func: (data: iFacebookInterfaces.iResult<iFacebookInterfaces.iFeed>) => void) {
        this.initHttp("/me/feed", "get").exec(function (data: iFacebookInterfaces.iResult<iFacebookInterfaces.iFeed>) {
            func(data);
        });
    }

    /**
     * Creates a new post
     * will return true if posting succeeded
     * 
     * @param {iFacebookInterfaces.iNewFeed} param (description)
     * @param {(data: { id: string }) => void} func the function which should retrieve the returned data
     */
    public post(param: iFacebookInterfaces.iNewFeed, func: (data: { id: string }) => void) {
        var http = this.initHttp("/me/feed", "post");
        http.addQueryObj(param);

        http.exec(function (data: { id: string }) {
            func(data);
        });
    }

    /**
     * Retrieves family members of the current user
     * 
     * @param {(data: iFacebookInterfaces.iResult<iFacebookInterfaces.iFamily>) => void} func the function which should retrieve the returned data
     */
    public family(func: (data: iFacebookInterfaces.iResult<iFacebookInterfaces.iFamily>) => void) {
        this.initHttp("/me/family", "get").exec(function (data: iFacebookInterfaces.iResult<iFacebookInterfaces.iFamily>) {
            func(data);
        });
    }

    /**
     * Retrieves checked in places by the current user
     * 
     * @param {(data: iFacebookInterfaces.iResult<iFacebookInterfaces.iTaggedPlaces>) => void} func the function which should retrieve the returned data
     */
    public places(func: (data: iFacebookInterfaces.iResult<iFacebookInterfaces.iTaggedPlaces>) => void) {
        this.initHttp("/me/places", "get").exec(function (data: iFacebookInterfaces.iResult<iFacebookInterfaces.iTaggedPlaces>) {
            func(data);
        });
    }
}

export = Facebook;