import request = require('request');
import ErrorHandler = require('../../ErrorHandler');

class HTTPClient {

    private _headers: Object;
    private url: string = null;
    private method: "get" | "post" | "put" | "delete" = null;
    private timeout = 4500;
    private _formData: Object = {};
    private _query: Object = {};
    private _oauth: Object = {};
    private stream: boolean = false;

    private http: request.Request;
    public constructor(url: string, method: "get" | "post" | "put" | "delete") {
        this._headers = {};

        this.url = url;
        this.method = method;

        this.addHeader('User-Agent', 'TileDSL');//some client requires the user-agent header to be set
    }

    public addHeader(key: string, value: string) {
        if (this._headers[key] == undefined)
            this._headers[key] = value;
        else
            console.log("Header for " + key + " already set");
    }

    public addData(key: string, value: string) {
        if (this._formData[key] == undefined)
            this._formData[key] = value;
        else
            console.log("Data for " + key + " is already set");
    }

    public addQuery(key: string, value: string) {
        if (this._query[key] == undefined)
            this._query[key] = value;
        else
            console.log("Data for " + key + " is already set");
    }

    public addQueryObj(obj: Object) {
        var self = this;
        Object.keys(obj).forEach(function (key) {
            self._query[key] = obj[key];
        });
    }

    private generateOptions(): Object {
        return {
            method: this.method,
            headers: this._headers,
            oauth: this._oauth,
            timeout: this.timeout,
            formData: this._formData,
            qs: this._query
        };
    }

    public exec(func: (data: JSON | any) => void) {
        var self = this;
        this.http = request(this.url, this.generateOptions(), function (error, response, body) {
            if (response == undefined)
                return ErrorHandler("No response");

            var contentType: Array<string> = response.headers['content-type'].split(';');

            if (error || response.statusCode != 200 || (contentType[0] != 'application/json' && contentType[0] != 'text/javascript')) {
                if (error != undefined && error.code == "ETIMEDOUT")
                    return ErrorHandler("TIMEOUT occurred for HTTP-call", this.url);
                return ErrorHandler("ERROR WITH HTTP: ",response,contentType);
            }

            try {//try parsing to body to JSON format
                var info = JSON.parse(body);
                return func(info);
            }
            catch (e) {
                return ErrorHandler("Couldn't parse body",e);
            }
        }).on('data', function (data) {
            if (self.stream) {
                try {
                    var tempD = JSON.parse(data.toString("utf8"));
                    return func(tempD);
                }
                catch (e) {
                    return ErrorHandler("Couldn't parse response data",data);
                }
            }

        }).on('response', function (response) {
            // unmodified http.IncomingMessage object 
            response.on('data', function (data) {

            });
        });
    }

    set header(headers: Object) {
        this._headers = headers;
    }

    set data(data: Object) {
        this._formData = data;
    }

    set queries(query: Object) {
        this._query = query;
    }

    set oauth(oauth: Object) {
        this._oauth = oauth;
    }

    public formData(data: Object): HTTPClient {
        this._formData = data;
        return this;
    }

    public headers(headers: Object): HTTPClient {
        this._headers = headers;
        return this;
    }

    public query(query: Object): HTTPClient {
        this._query = query;
        return this;
    }

    public streaming(stream: boolean = true) {
        this.stream = stream;
        return this;
    }
}
export = HTTPClient;