import HTTPClient = require('../HTTPClient');
import ErrorHandler = require('../../../ErrorHandler');

import iTwitterInterfaces = require('../Twitter/iTwitterInterfaces');

class Twitter {
    private http: HTTPClient = null;

    private baseurl: string = "https://api.twitter.com/1.1";
    private streamurl: string = "https://stream.twitter.com/1.1";
    private oauth: Object = null;


    /**
     * Creates an instance of Twitter.
     * Keys are optained by registering a new application at Twitter,
     * and obtaining keys from the user. The user needs to authorize 
     * the application however
     * @param {string} consumerKey 
     * @param {string} consumerKeySecret
     * @param {string} token
     * @param {string} tokenSecret 
     */
    public constructor(consumerKey: string, consumerKeySecret: string, token: string, tokenSecret: string) {
        this.oauth = {
            consumer_key: consumerKey,//Application keys
            consumer_secret: consumerKeySecret,
            token: token,//User keys
            token_secret: tokenSecret
        };
    }

    private initHttp(url: string, type: "get" | "post" | "put" | "delete"): HTTPClient {
        var http = new HTTPClient(this.baseurl + url, type);
        http.oauth = this.oauth;

        return http;
    }

    public tweet(param: iTwitterInterfaces.iTweet, func: (data: any) => void) {
        var http = this.initHttp("/statuses/update.json", "post");
        http.query(param);

        http.exec(function (data) {
            func(data);
        });
    }

    /**
     * Retrieves all followers for the user
     * @see https://dev.twitter.com/rest/reference/get/friends/ids
     * 
     * @param {iTwitterInterfaces.iFollowing} param to send with http request
     * @param {(data: any) => void} func the function which should retrieve the returned data
     */
    public followers(param: iTwitterInterfaces.iFollowing, func: (data: any) => void) {
        var http = this.initHttp("/friends/ids.json", "get");
        http.query(param);

        http.exec(function (data: iTwitterInterfaces.iFollowingResponse) {
            func(data);
        });
    }

    public following(param: iTwitterInterfaces.iFollowing, func: (data: any) => void) {
        var http = this.initHttp("/followers/ids.json", "get");
        http.oauth = this.oauth;
        http.query(param);

        http.exec(function (data: iTwitterInterfaces.iFollowingResponse) {
            func(data);
        });
    }

    public stream(param: iTwitterInterfaces.iStream, func: (data: iTwitterInterfaces.iStreamResponse) => void) {
        var http = new HTTPClient(this.streamurl + "/statuses/filter.json", "post");//use post to support longer queries
        var http = new HTTPClient(this.streamurl + "/statuses/sample.json", "get");//use post to support longer queries
        http.oauth = this.oauth;
        http.query(param);

        http.streaming(true).exec(function (data: iTwitterInterfaces.iStreamResponse) {
            func(data);
        });
    }

    public streamSample(func: (data: iTwitterInterfaces.iStreamResponse) => void) {
        var http = new HTTPClient(this.streamurl + "/statuses/sample.json", "get");//use post to support longer queries
        http.oauth = this.oauth;

        http.streaming(true).exec(function (data: iTwitterInterfaces.iStreamResponse) {
            func(data);
        });
    }

    //public firehose
}

export = Twitter;

//NOTES: For twitter to work:
// look at npm request example for twitter usage
// the TDE needs to retrieve the user token, send it to
// the TC for storage, which later can be used to do authorization 
// calls on behalf of the user
// TODO: test with my own access token to begin with