//Parameters required to create a new tweet
//@see https://dev.twitter.com/rest/reference/post/statuses/update
export interface iTweet {
    status: string, // the tweet message (max 140 characters)
    in_reply_to_status_id?: string, // The ID of an existing status that the update is in reply to
    possible_sensitive?: boolean, // If you upload Tweet media that might be considered sensitive content such as nudity, violence, or medical procedures, you should set this value to true. 
    lat?: number, // The latitude of the location this tweet refers to
    long?: number, // The longitude of the location this tweet refers to
    place_id?: string, // A place in the world @see https://dev.twitter.com/overview/api/places
    display_coordinates?: boolean, // Whether or not to put a pin on the exact coordinates a tweet has been sent from.
    trim_user?: boolean, // each Tweet returned in a timeline will include a user object including only the status authors numerical ID
    media_ids?: Array<string> // A list of media_ids to associate with the Tweet
}

export interface iFollowing {
    user_id?: string,//The ID of the user for whom to return results for.
    screen_name?: string, //The screen name of the user for whom to return results for.
    cursor?: number, // "page" to start returning result from
    stringify_ids?: boolean, //ids returned as strings instead
    count?: number //Specifies the number of IDs attempt retrieval of

}

export interface iFollowingResponse {
    ids: Array<number>,
    next_cursor: number,
    previous_cursor: number
}

export interface iStream {
    follow?: string, // A comma separated list of user IDs, indicating the users to return statuses for in the stream
    track?: string, //Keywords to track. Phrases of keywords are specified by a comma-separated list
    locations?: string, // Specifies a set of bounding boxes to track. comma separated lat,long
    delimited?: "length", // Specifies whether messages should be length-delimited. Indicates whether statuses should be delimited in the stream, so that users know how many bytes to read
    stall_warnings?: boolean // Specifies whether stall warnings should be delivered. notify user that they're falling behind on the stream, not able to retrieve data fast enough
}


export interface iStreamResponse {
    created_at: string,
    id: number,
    text: string,
    source: string, // html a link to
    truncated: boolean,
    in_reply_to_status_id: number,
    in_reply_to_user_id: number,
    in_reply_to_screen_name: string,

    user: {
        id: number, // user twitter id
        name: string,//user full name
        screen_name: string //user username
        location: string,
        url: string,
        description: string,
        protected: boolean,
        verified: boolean,
        followers_count: number,
        friends_count: number,
        listed_count: number,
        favourites_count: number,
        statuses_count: number,
        created_at: string,
        utc_offset: number,
        time_zone: string,
        geo_enabled: boolean,
        lang: string,
        contributors_enabled: boolean,
        is_translator: boolean,

        profile_background_color: string, //hex color (without hash)
        profile_background_image_url: string,
        profile_background_image_url_http: string,
        profile_background_tile: boolean,
        profile_link_color: string, //hex color (without hash)
        profile_sidebar_border_color: string, //hex color (without hash)
        profile_sidebar_fill_color: string, //hex color (without hash)
        profile_text_color: string, //hex color (without hash)
        profile_use_background_image: boolean,
        profile_image_url: string,
        profile_image_url_https: string,
        profile_banner_url: string,

        default_profile: boolean,
        default_profile_image: boolean,
        following: string,
        follow_request_sent: string,
        notifications: string
    },

    geo: string,
    coordinates: string,
    place: string,
    contributors: string,

    retweeted_status: Object, // same as appove

    is_quote_status: boolean,
    retweet_count: number,
    favorite_count: number,

    entities: {
        hashtags: Array<Object>,
        urls: Array<Object>,
        user_mentions: Array<Object>,
        symbols: Array<string>
    }

    favorited: boolean,
    retweeted: boolean,
    possible_sensitive: boolean,
    filter_level: string, //high,medium,low?
    lang: string, //language code
    timestamp_ms: string //timestamp in ms
}