import iWeatherInterfaces = require('./iWeatherInterfaces');




class CurrentWeather {
    private weather: iWeatherInterfaces.iCurrentWeather;

    public constructor(weather: iWeatherInterfaces.iCurrentWeather) {

        if (weather.rain != undefined)
            weather.rain._3h = weather.rain["3h"];
        if (weather.snow != undefined)
            weather.snow._3h = weather.snow["3h"];

        this.weather = weather;
    }

    get temperature(): number {
        return this.weather.main.temp;
    }

    get mintemp(): number {
        return this.weather.main.temp_min;
    }

    get maxtemp(): number {
        return this.weather.main.temp_max;
    }

    // percent of humidity
    get humidity(): number {
        return this.weather.main.humidity;
    }

    // percent of cloudiness
    get cloudiness(): number {
        if (this.weather.clouds != undefined)
            return this.weather.clouds.all;
        return null;
    }

    // Rain volume
    get rain(): number {
        if (this.weather.rain != undefined)
            return this.weather.rain._3h;
        return null;
    }

    // Snow volume
    get snow(): number {
        if (this.weather.snow != undefined)
            return this.weather.snow._3h;
        return null;
    }

    get weatherInfo(): iWeatherInterfaces.iCondition {
        return this.weather.weather[0];//will only retrieve one result when querying current
    }

    get sunrise(): Date {
        return new Date(this.weather.sys.sunrise * 1000);//convert from s to ms
    }

    get sunset(): Date {
        return new Date(this.weather.sys.sunset * 1000);
    }

    get time(): Date {
        return new Date(this.weather.dt * 1000);
    }

    get cityId(): number {
        return this.weather.id;
    }

    get cityName(): string {
        return this.weather.name;
    }



    public toString(): string {
        return JSON.stringify(this.weather);
    }
}

export = CurrentWeather;