import mongoose = require('mongoose');
import iTile = require('./iTile');
import iUser = require('./iUser');
import ErrorHandler = require('../../ErrorHandler');
import Models = require('../../mongoose/Models');
import EventEmitter = require('../Event/EventEmitter');

class Tile extends EventEmitter {
    private document: iTile;

    /**
     * Creates an instance of Tile.
     * 
     * @param {iTile} document (description)
     */
    constructor(document: iTile) {
        super();
        this.document = document;
    }

    /**
     * Look up Tile by its id
     * 
     * @static
     * @param {string} id of tile to search for
     * @param {(result: Tile) => void} cb callback function whill will retrieve the returned result
     */
    static findById(id: string, cb: (result: Tile) => void) {
        Models.Tile.findById(id).exec(function (error: any, tile: iTile) {
            if (error)
                return ErrorHandler(error);

            return cb(new Tile(tile));
        });
    }

    /**
     * Get id of the tile
     * 
     * @readonly
     * @type {string}
     */
    get id(): string {
        return this.document._id;
    }

    /**
     * whether the tile is active
     * 
     * @readonly
     * @type {boolean}
     */
    get active(): boolean {
        return this.document.active;
    }

    get name(): string {
        return this.document.name;
    }

    /**
     * retrive the state of the tile.
     * The last retrieved event or 
     * command sent to the device
     * 
     * @readonly
     * @type {Object}
     */
    get state(): Object {
        return this.document.state;
    }

}

export = Tile;