import Config = require('./Config');
import Animator = require('./Animator');

/**
 * Debugger, handles showing/hiding debugger console
 * and attaching new debug logs to registered dom
 * 
 * @export
 * @class Debugger
 */
class Debugger {
    private static dom: Element = null;
    private static debugText: Element = null;
    static active: boolean = false;

    /**
     * Creates an instance of Debugger.
     */
    public constructor() {

        Debugger.dom = document.getElementById(Config.DEBUG_DOM);
        Debugger.debugText = document.getElementById(Config.DEBUG_TEXT_DOM);


        //debug click handler
        document.getElementById(Config.DEBUG_BTN_DOM).onclick = function () {
            Debugger.active = (!Debugger.active);
            Debugger.toggleDebugger();
        }
    }

    public static toggleDebugger() {
        var self = this;
        if (this.active) {
            self.dom.className = "active";//show element before running animation
            Animator.animate(this.dom, "slideInUp");
        }
        else {

            Animator.animate(this.dom, "slideOutDown", function () {
                self.dom.className = "";//make sure that active class is removed AFTER animation
            });
        }
    }

    /**
     * Log output to emulator debugger
     * 
     * @static
     * @param {string} tag which log text should be associated with
     * @param {string} text the actual debug text
     */
    public static log(tag: string, text: string) {
        var tagDom = document.createElement("span");
        tagDom.className = "tag";
        tagDom.innerText = tag + " >";

        var log = document.createElement("div");
        log.appendChild(tagDom);
        log.appendChild(document.createTextNode(text));

        this.debugText.appendChild(log);
    }
}

export = Debugger;