/**
 * (description)
 * 
 * @class Functions
 */
class Functions {
    private static optionsMenuId = "extra-options-menu";

    /**
     * (description)
     * 
     * @static
     * @param {string} title (description)
     */
    static setTitle(title: string) {
        document.title = title;
    }

    /**
     * (description)
     * 
     * @static
     * @param {string} text (description)
     * @param {string} [icon="mif-info"] (description)
     */
    static setStatus(text: string, icon: string = "mif-info") {
        document.getElementById("status-text").innerText=text;
        document.getElementById("status-icon").className=icon;
    }

    /**
     * Initializes the options menu that is hidden in the header menu
     * This function will display the menu option, and run a given function
     * on click
     * 
     * @static
     * @param {Function} func function to run when a onclick event is triggered on dom-element
     */
    static initOptionsMenu(func: Function, display: string = "block") {
        var element = document.getElementById(Functions.optionsMenuId);

        element.style.display = display;
        element.onclick = function () { func(); }
    }


    /**
     * (description)
     * 
     * @static
     */
    static destroyOptionsMenu() {
        Functions.initOptionsMenu(null, "none");
    }

    /**
     * Will toggle a given charm, depending whether the charm is currently opened or closed.
     * The action parameter makes sure that the charm won't be open, if the desired action 
     * close.
     * 
     * @static
     * @param string id of charm to toggle
     * @param {string} [action=null] what action should be triggered on the charm [open|close]
     */
    static toggleCharm(id: string, action: string = null) {
        var charm = $("#" + id).data("charm");
        if (charm.element.data("opened") === true && action !== "open")
            charm.close();
        else if (action !== "close")
            charm.open();
    }

    /**
     * Opens a dialog, by specifying the dom-element's id
     * 
     * @static
     * @param string id of dom-element that should be a dialog
     */
    static openDialog(id: string) {
        var dialog = $("#" + id).data('dialog');
        dialog.open();
    }
}


export = Functions;