import Functions = require('../Functions');
import TextualEditorController = require('../Textual/TextualEditorController');
/**
 * VisualEditor controller, initializes a blockly editor
 * 
 * @class VisualEditorController
 */
class VisualEditorController {
    /**
     * static variable holding blockly instance. Made static to be able to "dispose" of it on  hash change
     * @see constructor
     * 
     * @static
     * @type {Blockly.Workspace}
     */
    public static blockly: Blockly.Workspace = null;

    /** Id of charm used in this page */
    private static buildCharmId = "visual-build-charm";

    /** Id of dialog to open when share button pressed from build-charm */
    private static shareDialog = "shareDialog";

    // Ids of elements defined in build-charm sidebar
    private static buildCharmElements = {
        "edit": "build-edit-code",
        "generate": "build-generate-code",
        "share": "build-share-code",
        "save": "build-save-code"
    };

    /**
     * Creates an instance of VisualEditorController. Initializes page which controller is registered too
     */
    constructor() {
        VisualEditorController.blockly = Blockly.inject('blocklyDiv',//div which should hold blockly editor
            {
                toolbox: document.getElementById('toolbox'),//toolbox defined as <xml>
                scrollbars: true,
                media: 'media/',//cursor and picture content needed by blockly
                zoom: {//zooming capabilities in the editor
                    enabled: true,
                    controls: true,
                    wheel: true,
                    maxScale: 2.5,
                    minScale: .1,
                    scaleSpeed: 1.1
                }

            });

        //make sure blockly is destructed on hash change (changing route)
        Spa.RouteMapper.runOnNextChange = function () {
            VisualEditorController.blockly.dispose();
            Functions.destroyOptionsMenu();
        }


        // document.getElementById("generateCode").onclick=function(){console.log("CLICKED");VisualEditorController.createCode();}

        Functions.initOptionsMenu(function () {
            Functions.toggleCharm(VisualEditorController.buildCharmId);
        });

        this.initBuildCharm();

    }

    /**
     * Creates JS code from the static workspace environment holding the Blockly instance
     * 
     * @static
     * @returns {string} the .js code created from blocks
     */
    public static createCode(): string {
        var code = Blockly.JavaScript.workspaceToCode(this.blockly);
        //console.log(Blockly.JavaScript);
        console.log("CODE HERE", code);
        return code;
    }

    /**
     * Initializes functions to be run when any buttons in the charm menu is pressed
     * 
     * @private
     */
    private initBuildCharm() {
        document.getElementById(VisualEditorController.buildCharmElements.edit).onclick = function () {
            TextualEditorController.storeCode(VisualEditorController.createCode());
            Spa.RouteMapper.setHash("textual/editor");
        }

        document.getElementById(VisualEditorController.buildCharmElements.generate).onclick = function () {
            VisualEditorController.createCode();
        }

        document.getElementById(VisualEditorController.buildCharmElements.share).onclick = function () {
            Functions.toggleCharm(VisualEditorController.buildCharmId, "close");
            Functions.openDialog(VisualEditorController.shareDialog);
        }

        document.getElementById(VisualEditorController.buildCharmElements.save).onclick = function () {

        }
    }
}

export = VisualEditorController;