declare module 'astring' {

    namespace astring {

        class Stream {
            write(str: string);
            toString(): string;
        }


        interface State {
            output?: Stream,
            generator?: DefaultGenerator,
            indent?: string, //default '\t'
            lineEnd?: string, //default '\n'
            indentLevel?: number, //default 0
            writecomments?: boolean, //default false
            noTrailingSemicolon?: boolean //default false
        }

        export interface DefaultGenerator {
            Program(node: ESTree.Program, state: State);
            BlockStatement(node: ESTree.BlockStatement, state: State);
            EmptyStatement(node: ESTree.EmptyStatement, state: State);
            ExpressionStatement(node: ESTree.ExpressionStatement, state: State);
            IfStatement(node: ESTree.IfStatement, state: State);
            LabeledStatement(node: ESTree.LabeledStatement, state: State);
            BreakStatement(node: ESTree.BreakStatement, state: State);
            ContinueStatement(node: ESTree.ContinueStatement, state: State);
            WithStatement(node: ESTree.WithStatement, state: State);
            SwitchStatement(node: ESTree.SwitchStatement, state: State);
            ReturnStatement(node: ESTree.ReturnStatement, state: State);
            ThrowStatement(node: ESTree.ThrowStatement, state: State);
            TryStatement(node: ESTree.TryStatement, state: State);
            WhileStatement(node: ESTree.WhileStatement, state: State);
            DoWhileStatement(node: ESTree.DoWhileStatement, state: State);
            ForStatement(node: ESTree.ForStatement, state: State);
            ForInStatement(node: ESTree.ForInStatement, state: State);
            ForOfStatement(node: ESTree.ForOfStatement, state: State);
            DebuggerStatement(node: ESTree.DebuggerStatement, state: State);
            FunctionDeclaration(node: ESTree.FunctionDeclaration, state: State);
            FunctionExpression(node: ESTree.FunctionExpression, state: State);
            VariableDeclaration(node: ESTree.VariableDeclaration, state: State);
            VariableDeclarator(node: ESTree.VariableDeclarator, state: State);
            ClassDeclaration(node: ESTree.ClassDeclaration, state: State);
            ImportDeclaration(node: ESTree.ImportDeclaration, state: State);
            ExportDefaultDeclaration(node: ESTree.ExportDefaultDeclaration, state: State);
            ExportNamedDeclaration(node: ESTree.ExportNamedDeclaration, state: State);
            ExportAllDeclaration(node: ESTree.ExportAllDeclaration, state: State);
            MethodDefinition(node: ESTree.MethodDefinition, state: State);
            ClassExpression(node: ESTree.ClassExpression, state: State);
            ArrowFunctionExpression(node: ESTree.ArrowFunctionExpression, state: State);
            ThisExpression(node: ESTree.ThisExpression, state: State);
            Super(node: ESTree.Super, state: State);
            RestElement(node: ESTree.RestElement, state: State);
            SpreadElement(node: ESTree.SpreadElement, state: State);
            YieldExpression(node: ESTree.YieldExpression, state: State);
            TemplateLiteral(node: ESTree.TemplateLiteral, state: State);
            TaggedTemplateExpression(node: ESTree.TaggedTemplateExpression, state: State);
            ArrayExpression(node: ESTree.ArrayExpression, state: State);
            ArrayPattern(node: ESTree.ArrayPattern, state: State);
            ObjectExpression(node: ESTree.ObjectExpression, state: State);
            Property(node: ESTree.Property, state: State);
            ObjectPattern(node: ESTree.ObjectPattern, state: State);
            SequenceExpression(node: ESTree.SequenceExpression, state: State);
            UnaryExpression(node: ESTree.UnaryExpression, state: State);
            UpdateExpression(node: ESTree.UpdateExpression, state: State);
            AssignmentExpression(node: ESTree.AssignmentExpression, state: State);
            AssignmentPattern(node: ESTree.AssignmentPattern, state: State);
            BinaryExpression(node: ESTree.BinaryExpression, state: State);
            LogicalExpression(node: ESTree.LogicalExpression, state: State);
            ConditionalExpression(node: ESTree.ConditionalExpression, state: State);
            NewExpression(node: ESTree.NewExpression, state: State);
            CallExpression(node: ESTree.CallExpression, state: State);
            MemberExpression(node: ESTree.MemberExpression, state: State);
            MetaProperty(node: ESTree.MetaProperty, state: State);
            Identifier(node: ESTree.Identifier, state: State);
            Literal(node: ESTree.Literal, state: State);
        }

        export var defaultGenerator: DefaultGenerator;
    }

    function astring(node: ESTree.Node, options: astring.State): string;

    export = astring;
}